import numpy as np
import glob


def read_arguments():
    # Using optparse library to get command line options and arguments
    from optparse import OptionParser

    # Reading the command line arguments and
    # Creating the parser
    parser = OptionParser()
    # Adding options
    # Option for generating simplified output
    # parser.add_option("-r", help="Reference File", dest="reference", type="string", default="year_01")
    parser.add_option("-f", help="File to evaluate", dest="to_evaluate", type="string")

    # Parse arguments and options
    (options, args) = parser.parse_args()
    print(options)

    _evaluated_filename_pattern = options.to_evaluate

    return _evaluated_filename_pattern


def load_pkl(_filename):
    import pickle as pkl

    return pkl.load(open(_filename, "rb"))


def plot_result(result=None, output_folder=None, limits=None):
    import matplotlib
    import matplotlib.pyplot as plt

    matplotlib.use('Agg', warn=False, force=True)
    # The result file has to be a dictionary containing the results from an experiment

    # TODO: What a mess, should replace this
    # Path to the ensemble members used to create the limits file
    ensemble_members_path = "/home/oriol/tmp/analysis_configuration_files/member_*_RMSE.pkl"

    # Get filenames
    filenames = glob.glob(ensemble_members_path)

    # Load members data
    members = []
    for filename in filenames:
        members.append(load_pkl(filename))

    if result is not None:
        grids = result.keys()
        variables = {}
        for grid in grids:
            variables[grid] = result[grid].keys()
    else:
        # Pick one case to extract grid and variable names
        example_file = members[0]
        grids = example_file.keys()
        variables = {}
        for grid in grids:
            variables[grid] = example_file[grid].keys()
        del example_file

    # Start plotting
    for grid in grids:
        for variable in variables[grid]:
            if variable.count("time"):
                continue
            # Plot limits
            try:
                y0 = limits[grid][variable][0]
                y1 = limits[grid][variable][1]
                x0 = range(len(y0))
                plt.fill_between(x0, y0, y1, alpha=.2)
            except KeyError:
                print("No limits for %s in %s" % (variable, grid))

            # Plot ensemble average and standard deviation
            var_data = np.array([m[grid][variable] for m in members])
            average = np.average(var_data, axis=0)
            std = np.std(var_data, axis=0)
            x0 = range(len(average))
            plt.fill_between(x0, average - std, average + std, color="green", alpha=.2)
            plt.plot(average, "--", color="green")

            # Plot each member
            for member in members:
                plt.plot(member[grid][variable], color="gray", alpha=.2)

            # Plot result being evaluated
            if result is not None:
                vdata = result[grid][variable]
                plt.plot(vdata, color="red")
                plt.scatter(range(len(vdata)), vdata, color="red")


            plt.title(variable)
            plt.savefig("%s/%s.png" % (output_folder, variable))
            plt.clf()
    from AutoRPE.EvaluationScripts.gallery_creator import create_viewer
    create_viewer(output_folder)


if __name__ == "__main__":
    plot_result(result=None, output_folder="Figures")
