from os.path import join
from AutoRPE.UtilsRPE.Functions import in_which_block, remove_comments, load_vault, \
                                       replace_variable_exact_match
from AutoRPE.UtilsRPE.Classes import Current_block, ImplementRPE
from AutoRPE.UtilsRPE.CodeTools import expand_sources, merge_lines
from AutoRPE.UtilsRPE.track_dependecy import retrieve_real_in_line
from AutoRPE.UtilsRPE.Error import ProcedureNotFound
import warnings
import re


def find_call_in_original_file(routine, argument, _file, target_type="dp", fix=False):
    # Add a cast to the proper type
    new_argument = "REAL(%s, %s)" % (argument, target_type)
    # Fix the pattern to account for regex rules
    pattern = "%s####%s" % (routine, argument)
    pattern = pattern.replace("(", "\(")
    pattern = pattern.replace(")", "\)")
    pattern = pattern.replace("+", "\+")
    pattern = pattern.replace("*", "\*")
    pattern = pattern.replace("####", ".*")

    # Initialize found to False
    found = False

    # Open file
    with open(_file, "r") as f:
        lines = [l for l in f]
        for line_index, line in enumerate(lines):
            if line.lower().count("subroutine") or line.lower().count("function"):
                continue
            m = re.search(pattern, line, re.I)
            if m:
                # The args have already been fixed
                if line.count(new_argument) > 0:
                    continue
                found = True
                new_line = replace_variable_exact_match(argument, new_argument, line)
                lines[line_index] = new_line
                break
            else:
                if line.count(routine):
                    # The line contains the routine name but the pattern doesn't match
                    line = remove_comments(lines[line_index]).strip()
                    # Check if it is a call with split args
                    if line and line.strip()[-1] == "&":
                        line = merge_lines(_file, line_index, fix_in_file=False)
                        m = re.search(pattern, line, re.I)
                        if m:
                            # It merge the split call in one single line
                            merge_lines(_file, line_index, fix_in_file=True)
                            # Now the function can properly fix the call
                            return find_call_in_original_file(routine, argument, _file, target_type, fix)

    # In case we found the call overwrite the file with the fix
    # Otherwise return False
    if found:
        if fix:
            code = "".join(lines).strip()
            with open(_file, "w") as output:
                output.write(code)
        return True
    else:
        return False


def code_check_file(filename, vault, fix=False):
    try:
        current_block = Current_block(vault[filename.split("/")[-1].replace(".f90", "")].main())
    except AttributeError:
        warnings.warn("No module %s" % filename)
        return
    f = ImplementRPE(filename, None)

    f.pre_process()
    # Loop through lines
    for line in f.lines:
        # Returns all REAL var on the line which corresponding dummy argument has different type
        procedure_tf, arguments_proc_tf, dummy_var_type_tf, dummy_var_intent_tf = \
            retrieve_real_in_line(line, vault, current_block=current_block, check=True)
        # If something was found check if can be fixed
        if [procedure_tf, arguments_proc_tf, dummy_var_type_tf] != [None, None, None]:
            for p_tf, ap_tf, dv_tf, dv_i_tf in zip(procedure_tf, arguments_proc_tf,
                                                   dummy_var_type_tf, dummy_var_intent_tf):
                types = {"single": "sp", "double": "dp"}
                try:
                    target_type = types[dv_tf]
                except KeyError:
                    continue
                # If the intent is inout or out it can not be fixed with a cast
                if fix:
                    if dv_i_tf.count("out"):
                        fix_this = False
                    else:
                        fix_this = True
                else:
                    fix_this = False

                fixed = find_call_in_original_file(p_tf, ap_tf, filename,
                                                   target_type=target_type, fix=fix_this)
                if fixed:
                    if fix:
                        print("Was fixed!")
                    else:
                        print("Can be fixed!")
                else:
                    print("Couldn't be fixed")

        else:
            # If not, look if the code is entering or exiting a routine block
            current_block = in_which_block(line, current_block, vault=vault)


if __name__ == "__main__":
    # Getting Command line options
    from optparse import OptionParser

    parser = OptionParser()
    parser.add_option("-v", "--vault", dest="vault", default=None, metavar="FILE")
    parser.add_option("-s", "--input-source", dest="input_source", default=None, metavar="FILE")
    parser.add_option("-w", "--working-precision", dest="working_precision", default="single")
    parser.add_option("-t", "--target-precision", dest="target_precision", default="double")
    parser.add_option("-f", "--fix-dependencies", dest="fix_dependencies", default=False, action="store_true")

    (options, args) = parser.parse_args()
    vault_path = options.vault
    wp = options.working_precision
    target_precision = options.target_precision
    path_to_input_sources = options.input_source
    fix_dependencies = options.fix_dependencies

    if path_to_input_sources is None or vault_path is None:
        print("\nArguments -s/--input-source and -v/--vault are mandatory\n")
        parser.print_help()
        exit(1)

    # Loading vault
    vault = load_vault(vault_path)

    # Find all files in sources
    all_sources = expand_sources(path_to_input_sources)
    # Process Sources!
    total = len(all_sources)
    for index, filename in enumerate(all_sources):
        code_check_file(filename, vault, fix=fix_dependencies)
