import unittest


class TestFunctionMethods(unittest.TestCase):

    def test_type_of_real_cast_with_double_specification(self):
        from AutoRPE.UtilsRPE.Functions import type_of_real_cast
        string = 'real((ssh(:,:,kmm)+ssh_ref)*tmask(:,:,1),dp)'
        self.assertEqual(type_of_real_cast(string), 'double')

    def test_type_of_real_cast_with_single_specification(self):
        from AutoRPE.UtilsRPE.Functions import type_of_real_cast
        string = 'real((ssh(:,:,kmm)+ssh_ref)*tmask(:,:,1),sp)'
        self.assertEqual(type_of_real_cast(string), 'single')

    def test_type_of_real_cast_no_specification(self):
        from AutoRPE.UtilsRPE.Functions import type_of_real_cast
        from AutoRPE.UtilsRPE.Functions import working
        string = 'real((ssh(:,:,kmm)+ssh_ref)*tmask(:,:,1))'
        self.assertEqual(type_of_real_cast(string), working)

    def test_separate_line_container_and_arguments_1(self):
        from AutoRPE.UtilsRPE.Functions import separate_line_container_and_arguments
        string = 'write(numicb,*) \'bergstep \',nktberg,\' send ns: \', ibergs_to_send_n, ibergs_to_send_s'

        line_container, arguments = separate_line_container_and_arguments(string=string)
        actual_arguments = ["'bergstep '", 'nktberg', "' send ns: '", ' ibergs_to_send_n', ' ibergs_to_send_s']
        actual_line_container = 'write(numicb,*) ###TOREPLACE###'

        self.assertTrue(line_container == actual_line_container and actual_arguments == arguments)

    def test_separate_line_container_and_arguments_2(self):
        from AutoRPE.UtilsRPE.Functions import separate_line_container_and_arguments
        string = "WRITE(cl_asmbkg, FMT='(A,\".nc\")' ) TRIM( c_asmbkg )"

        line_container, arguments = separate_line_container_and_arguments(string=string)
        actual_arguments = ['TRIM( c_asmbkg )']
        actual_line_container = 'WRITE(cl_asmbkg, FMT=\'(A,".nc")\' ) ###TOREPLACE###'

        self.assertTrue(line_container == actual_line_container and actual_arguments == arguments)

    def test_separate_line_container_and_arguments_3(self):
        from AutoRPE.UtilsRPE.Functions import separate_line_container_and_arguments
        string = "IF(lwp) WRITE(numout,'(a,i6,a,i2,a,i2,a,i8,a,i8,a,i8,a,i8)')                   ' =======>> 1/2 time step before the start of the run DATE Y/M/D = ',                   nyear, '/', nmonth, '/', nday, '  nsec_day:', nsec_day, '  nsec_monday:', nsec_monday, '                   nsec_month:', nsec_month , '  nsec_year:' , nsec_year"
        line_container, arguments = separate_line_container_and_arguments(string=string)
        condition = arguments[0].count("WRITE") == 0
        self.assertTrue(condition)

    def test_separate_line_container_and_arguments_4(self):
        from AutoRPE.UtilsRPE.Functions import separate_line_container_and_arguments
        string='IF( lwriter ) WRITE(numtime,*) \'        ===> W A R N I N G: \''
        line_container, arguments = separate_line_container_and_arguments(string=string)
        condition = arguments[0].count("WRITE") == 0
        self.assertTrue(condition)

    def test_separate_line_container_and_arguments_4(self):
        from AutoRPE.UtilsRPE.Functions import separate_line_container_and_arguments
        string='IF (ln_grid_search_lookup)WRITE(numout,*) \'      Grid search lookup file header                cn_gridsearchfile = \', cn_gridsearchfile'
        line_container, arguments = separate_line_container_and_arguments(string=string)
        condition = arguments[0].count("WRITE") == 0
        self.assertTrue(condition)

    def test_dimension_of_contents_1(self):
        from AutoRPE.UtilsRPE.Functions import dimension_of_contents
        self.assertEqual(dimension_of_contents("1.0"),0)

    def test_dimension_of_contents_2(self):
        from AutoRPE.UtilsRPE.Functions import dimension_of_contents
        string = "real(  ( ( z3d(:,:,:) - e3t_0(:,:,:) ) / e3t_0(:,:,:) * 100 * tmask(:,:,:) ) ** 2  )"
        dimension = dimension_of_contents(string)
        self.assertEqual(dimension, 3)

    def test_dimension_of_contents_3(self):
        from AutoRPE.UtilsRPE.Functions import dimension_of_contents
        string = "e1e2t(:,:) * zfwf(:,:)"
        dimension = dimension_of_contents(string)
        self.assertEqual(dimension, 2)

    def test_add_val(self):
        # Need to be sure that same strings are not replaced twice
        pass


if __name__ == '__main__':
    unittest.main()